@@ -1,4 +1,6 @@
 ## Simulate data from an AFT model
+library(afttest)
+library(survival)
 datgen <- function(n = 100) {
   z1 <- rbinom(n, 1, 0.5)
   z2 <- rnorm(n)
@@ -10,12 +12,6 @@ datgen <- function(n = 100) {
 }
 set.seed(0)
 simdata <- datgen(n = 20)
-
-X <- simdata$Time
-D <- simdata$status
-z1 <- simdata$z1
-z2 <- simdata$z2
-
-result = afttest(Surv(X, D) ~ z1 + z2, testType="link", eqType="mns")
-print(result)
+result <- afttest(Surv(Time, status) ~ z1 + z2, 
+                  data = simdata, testType="link", eqType="mns")
 summary(result)
\ No newline at end of file
